@@ -16,7 +16,7 @@ use IO::File ();
 use IO::Prompt ();
 use Text::Balanced ();
 
-our $VERSION = '0.47_05';
+our $VERSION = '0.47_06';
 
 use constant LEADCHAR => '* ';
 
@@ -400,7 +400,7 @@ sub _parse_makefile {
         }
 
         $self->{parse}{makefile} = ''
-          if $self->{parse}{makefile} !~ /\w/;
+          unless $self->{parse}{makefile} =~ /\w/;
     }
 
     $self->_debug(LEADCHAR."Leaving parse\n\n", 'no_wait');
@@ -421,7 +421,7 @@ sub _parse_regexps {
     my $found_string = qr/^
                             \s* 
                             ['"]? (\w+) ['"]?
-                            \s* => \s* [^ \{ \[ ]
+                            \s* => \s* (?![ \{ \[ ])
                             ['"]? ([\$ \@ \% \< \> \( \) \\ \/ \- \: \. \w]+.*?) ['"]?
                             ,? ([^\n]+ \# \s+ \w+ .*?)? \n
                        /sx;
@@ -449,7 +449,8 @@ sub _parse_process_string {
     $value   ||= '';
     $comment ||= '';
 
-    $value =~ tr/['"]//d;
+    $value =~ s/^['"]//;
+    $value =~ s/['"]$//;
 
     $self->{parse}{makeargs}{$arg} = $value;
     push @{$self->{parse}{histargs}}, $arg;
@@ -528,13 +529,14 @@ sub _parse_process_code {
         $self->{parse}{makefile} .= "\n"
           unless $self->{parse}{makefile} =~ /\n$/s;
 
-        while ($self->{parse}{makefile} =~ /\G(\s*?\#.*?(,|\.)?\n)/cgs) {
+        while ($self->{parse}{makefile} =~ /\G(\s*?\#.*?\n)/cgs) {
             $comment .= $1;
         }
 
         $comment ||= '';
 
-        $self->{parse}{makefile} =~ s/$comment//;
+        my $quoted_comment = quotemeta $comment;
+        $self->{parse}{makefile} =~ s/$quoted_comment//s; 
 
         my @comment;
 
@@ -689,7 +691,7 @@ sub _read_makefile {
     my $self = shift;
 
     my $makefile = File::Slurp::read_file($self->{Config}{Makefile_PL});
-    $makefile =~ s/^(.*)\&?WriteMakefile\s*?\(\s*(.*?)\s*\)\s*?;(.*)$/$2/s;
+    $makefile =~ s/^(.*?)\&?WriteMakefile\s*?\(\s*(.*?)\s*\)\s*?;(.*)$/$2/s;
 
     my $makecode_begin = $1;
     my $makecode_end   = $3;
@@ -903,42 +905,70 @@ sub _write {
 sub _compose_header {
     my $self = shift;
 
-    my ($comments_header, $code_header) = ('','');
+    my ($comments_header, $code_header);
+
     my $note = '# Note: this file has been initially generated by ' . __PACKAGE__ . " $VERSION";
     my $pragmas = "use strict;\nuse warnings;\n";
 
-    if (defined $self->{make_code}{begin}) {
-        $self->_do_verbose(LEADCHAR."Removing ExtUtils::MakeMaker as dependency\n");
+    # Warnings are thrown for chomp() & regular expressions when enabled
+    no warnings 'uninitialized';
 
+    if (defined $self->{make_code}{begin} || defined $self->{make_code}{end}) {
+        # Removing ExtUtils::MakeMaker dependency
+        $self->_do_verbose(LEADCHAR."Removing ExtUtils::MakeMaker as dependency\n");
         $self->{make_code}{begin} =~ s/[ \t]*(?:use|require)\s+ExtUtils::MakeMaker\s*;//;
 
+        # Mapping (prompt|Verbose) calls to Module::Build ones
         if ($self->{make_code}{begin} =~ /(?:prompt|Verbose)\s*\(/s) {
             my $regexp = qr/^(.*?=\s*)(prompt|Verbose)\s*?\(['"](.*)['"]\);$/;
 
-            for my $var (qw(begin end)) {
+            foreach my $var (qw(begin end)) {
                 while ($self->{make_code}{$var} =~ /$regexp/m) {
                     my $replace = $1 . 'Module::Build->' . $2 . '("' . $3 . '");';
                     $self->{make_code}{$var} =~ s/$regexp/$replace/m;
                 }
             }
         }
+
+        # Removing Module::Build::Compat Note
         if ($self->{make_code}{begin} =~ /Module::Build::Compat/) {
             $self->_do_verbose(LEADCHAR."Removing Module::Build::Compat Note\n");
             $self->{make_code}{begin} =~ s/^\#.*Module::Build::Compat.*?\n//s;
         }
-        # Removing pragmas quietly here to ensure that they'll be inserted after
-        # an eventually appearing version requirement.
+
+        # Removing customized MakeMaker subs
+        my $has_MM_sub    = qr/sub MY::/;
+        my $MM_sub_prefix = 'MY::';
+
+        foreach my $var (qw(begin end)) {
+            if ($self->{make_code}{$var} =~ $has_MM_sub) {
+                foreach my $sub (_extract_sub($self->{make_code}{$var}, $MM_sub_prefix)) {
+                    my $quoted_sub = quotemeta $sub;
+                    my ($subname)  = $sub =~ /sub.*?\s+(.*?)\s*\{/;
+
+                    $self->{make_code}{$var} =~ s/$quoted_sub\n//;
+                    $self->_do_verbose(LEADCHAR."Removing sub: '$subname'\n");
+                }
+            }
+        }
+
+        # Removing strict & warnings pragmas quietly here to ensure that they'll
+        # be inserted after an eventually appearing version requirement.
         $self->{make_code}{begin} =~ s/[ \t]*use\s+(?:strict|warnings)\s*;//g;
+
+        # Saving the shebang (interpreter) line
         while ($self->{make_code}{begin} =~ s/^(\#\!?.*?\n)//) {
             $comments_header .= $1;
         }
         chomp $comments_header;
 
-        while ($self->{make_code}{begin} =~ /(?:use|require)\s+.*?;/) {
-            $self->{make_code}{begin} =~ s/^\n?(.*?;)//s;
+        # Grabbing use & require statements
+        while ($self->{make_code}{begin} =~ /^(?:use|require)\s+(?:[a-z]|[\d\.\_])+?\s*;/m) {
+            $self->{make_code}{begin} =~ s/^\n*(.*?;)//s;
             $code_header .= "$1\n";
         }
 
+        # Adding strict & warnings pragmas
         $self->_do_verbose(LEADCHAR."Adding use strict & use warnings pragmas\n");
 
         if ($code_header =~ /(?:use|require)\s+\d\.[\d_]*\s*;/) { 
@@ -948,10 +978,12 @@ sub _compose_header {
         }
         chomp $code_header;
 
+        # Removing leading & trailing newlines
         1 while $self->{make_code}{begin} =~ s/^\n//;
         chomp $self->{make_code}{begin} while $self->{make_code}{begin} =~ /\n$/s;
     }
 
+    # Constructing the Build.PL header
     $self->{Data}{begin} = $comments_header || $code_header
       ? ($comments_header  =~ /\w/ ? "$comments_header\n" : '') . "$note\n" .
         ($code_header =~ /\w/ ? "\n$code_header\n\n" : "\n") .
@@ -959,6 +991,36 @@ sub _compose_header {
       : "$note\n\n" . $self->{Data}{begin};
 }
 
+# Albeit Text::Balanced exists, extract_tagged() and friends
+# were (or I?) unable to extract subs.
+sub _extract_sub {
+    my ($text, $pattern) = @_;
+
+    my ($quoted_pattern, %seen, @sub, @subs);
+
+    $quoted_pattern = quotemeta $pattern;
+
+    foreach my $line (split /\n/, $text) {
+        if ($line =~ /^sub $quoted_pattern\w+/s ||
+            $line =~ /^\{/)                        { $seen{begin} = 1 }
+        if ($seen{begin} && $line =~ /^\s*}/)      { $seen{end}   = 1 }
+
+        if ($seen{begin} || $seen{end}) {
+            push @sub, $line;
+        } else {
+            next;
+        }
+
+        if ($seen{end}) {
+            push @subs, join "\n", @sub;
+            @sub = ();
+            @seen{qw(begin end)} = (0,0);
+        }
+    }
+
+    return @subs;
+}
+
 sub _write_begin {
     my $self = shift;
 
